jls.loader.provide('jls.util.Formatter');

jls.loader.require('jls.util.Resource');
jls.loader.require('jls.util.Locale');

/**
 * @class This class provides a printf like String formatter.
 */
jls.util.Formatter = jls.lang.Class.create({});

/*
 * %[argument_index$][flags][width][.precision]conversion
 */
Object.extend(jls.util.Formatter, /** @lends jls.util.Formatter */
{
    _lineSeparator : _native.core.properties['line.separator'],
    _resources : {},
    getResource : function(language) {
        if (language in jls.util.Formatter._resources) {
            return jls.util.Formatter._resources[language];
        }
        var res = jls.util.Resource.load('jls/util/formatter', language);
        var resource = {
                dayFullNames: res.get('dayFullNames').split(','),
                monthFullNames: res.get('monthFullNames').split(',')
        };
        return jls.util.Formatter._resources[language] = resource;
    },
    /**
     * Returns a padded string.
     *
     * @param {String} s The string to pad.
     * @param {String} l The padding length.
     * @param {String} c The padding character.
     * @param {String} d The direction, true for left padding.
     * @returns {String} The padded string.
     */
    pad : function(s, l, c, d) {
        if ((! l) || (l < 1)) {
            return '';
        }
        s = s.toString();
        if (s.length >= l) {
            return s;
        }
        c = c || ' ';
        d = d || false;
        var p = '';
        for (var i = s.length; i < l; i++) {
            p += c;
        }
        if (d) {
            s = p + s;
        } else {
            s += p;
        }
        return s;
    },
    /**
     * Returns a formatted string.
     *
     * @param {String} [locale] The locale to use to format.
     * @param {String} format The format string.
     * @returns {String} The formatted string.
     */
    format : function() {
        var args = Array.from(arguments);
        var locale;
        if (args[0] instanceof jls.util.Locale) {
            locale = args.shift();
        } else {
            locale = jls.util.Locale.getDefault();
        }
        var format = args.shift();
        if (jls.logger.isTraceEnabled()) {
            jls.logger.trace('format(' + locale + ', "' + format + '", ' + args.toJSON() + ')');
        }
        var argi = 0;
        return format.replace(/%(<|[0-9]+\$){0,1}([+\- 0#]+){0,1}([0-9]+){0,1}(?:\.([0-9]+)){0,1}([dinsux%]|[tT][yYmedHIklMSLAB])/g,
        function(str, index, flags, width, precision, specifier) {
            switch (specifier) {
            case 'n': return jls.util.Formatter._lineSeparator;
            case '%': return specifier;
            }
            var padLeft = false;
            var padChar = ' ';
            var ai = index == '<' ? argi : (index ? parseInt(index, 10) : ++argi);
            if ((ai < 1) || (ai > args.length)) {
                throw new jls.lang.Exception('Missing argument ' + ai);
            }
            for (var i = 0; flags && (i < flags.length); i++) {
                var flag = flags.charAt(i);
                switch (flag) {
                case '+':
                    padLeft = false;
                    break;
                case '-':
                    padLeft = true;
                    break;
                case '-':
                case '0':
                    padChar = flag;
                    break;
                }
            }
            width = width ? parseInt(width, 10) : 0;
            precision = precision ? parseInt(precision, 10) : 0;
            var arg = args[ai - 1];
            if ((typeof arg == 'undefined') || (arg == null)) {
                throw new jls.lang.Exception('Missing argument ' + ai);
            }
            // format argument
            switch (specifier.charAt(0)) {
            case 'd':
            case 'i':
            case 's':
            case 'u':
                break;
            case 'x':
                arg = arg.toString(16);
                break;
            case 't':
            case 'T':
                var p = 2;
                switch (specifier.charAt(1)) {
                case 'y':
                    arg = arg.getYear();
                    break;
                case 'Y':
                    p = 0;
                    arg = arg.getFullYear();
                    break;
                case 'm':
                    arg = arg.getMonth() + 1;
                    break;
                case 'e':
                    p = 0;
                case 'd':
                    arg = arg.getDate();
                    break;
                case 'k':
                    p = 0;
                case 'H':
                    arg = arg.getHours();
                    break;
                case 'l':
                    p = 0;
                case 'I':
                    arg = ((arg.getHours() + 11) % 12 + 1);
                    break;
                case 'M':
                    arg = arg.getMinutes();
                    break;
                case 'S':
                    arg = arg.getSeconds();
                    break;
                case 'L':
                    p = 3;
                    arg = arg.getMilliseconds();
                    break;
                case 'A':
                    p = 0;
                    arg = jls.util.Formatter.getResource(locale.language).dayFullNames[arg.getDay()];
                    break;
                case 'B':
                    p = 0;
                    arg = jls.util.Formatter.getResource(locale.language).monthFullNames[arg.getMonth()];
                    break;
                default:
                    throw new jls.lang.Exception('Invalid date specifier: ' + specifier.charAt(1));
                }
                if (p > 0) {
                    arg = jls.util.Formatter.pad(arg, p, '0', true);
                }
                break;
            default:
                throw new jls.lang.Exception('Invalid specifier: ' + specifier.charAt(0));
            }
            arg = arg.toString();
            if (width && (arg.length < width)) {
                arg = jls.util.Formatter.pad(arg, width, padChar, padLeft);
            }
            if (precision && (arg.length > precision)) {
                if (padLeft) {
                    arg = arg.substr(-precision, precision);
                } else {
                    arg = arg.substr(0, precision);
                }
            }
            return arg;
        });
    }
});
